<?php
/**
 *
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\RuleDayFromBirthday\Controller\Adminhtml\Promo\Quote;


use Magento\Backend\App\Action;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultFactory;
use Magento\SalesRule\Model\Service\CouponManagementService;
use Magento\SalesRule\Model\Data\CouponGenerationSpec;
use Magento\SalesRule\Model\Coupon;
use Magento\SalesRule\Helper\Coupon as HelperCoupon;
use Magento\SalesRule\Api\CouponRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Birthdaycoupon extends Action
{
    protected $resultFactory;
    protected $couponManagement;
    protected $generator;
    protected $couponRepository;
    protected $coupon;
    protected $scopeConfig;
    const RANGE = 'filoblu_ruledayfrombirthday_section/birthday_coupon_id/validation_range_text';
    const START_DATE ='filoblu_ruledayfrombirthday_section/birthday_coupon_id/start_date_id';


    /**
     * Birthdaycoupon constructor.
     * @param Action\Context $context
     * @param JsonFactory $resultFactory
     * @param CouponGenerationSpec $couponGenerationSpec
     * @param CouponManagementService $couponManagementService
     * @param CouponRepositoryInterface $couponRepository
     * @param Coupon $coupon
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(Action\Context $context, JsonFactory $resultFactory,CouponGenerationSpec $couponGenerationSpec,
                                CouponManagementService $couponManagementService, CouponRepositoryInterface $couponRepository,
                                Coupon $coupon,
                                ScopeConfigInterface $scopeConfig
    )
    {
        $this->resultFactory = $resultFactory;
        $this->couponManagement = $couponManagementService;
        $this->generator = $couponGenerationSpec;
        $this->couponRepository = $couponRepository;
        $this->coupon = $coupon;
        $this->scopeConfig = $scopeConfig;

        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Json|\Magento\Framework\Controller\ResultInterface|void
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function execute()
    {

        if (!$this->getRequest()->isAjax()) {
            $this->_forward('noroute');
            return;
        }

        $rule_id = $this->getRequest()->getParam('rule_id');
        $qty = $this->getRequest()->getParam('qty');
        $length =$this->getRequest()->getParam('length');
        $format = HelperCoupon::COUPON_FORMAT_ALPHANUMERIC;
        $prefix = $this->getRequest()->getParam('prefix');
        $suffix = $this->getRequest()->getParam('suffix');

        if(!$qty || !$rule_id)
        {
            return;
        }

        if($this->scopeConfig->getValue(self::START_DATE)!='')
        {
            $startDate = $this->scopeConfig->getValue(self::START_DATE);
        }
        else
        {
            $startDate = date('Y-m-d H:i:s', mktime(0,0,0,1,1,date('Y')));
        }


        for ( $i=0; $i<$qty; $i++)
        {
            $this->generator->setLength($length)->setPrefix($prefix)
                ->setSuffix($suffix)
                ->setFormat($format)
                ->setRuleId($rule_id)
                ->setQuantity(1);
            $range = (int) $this->scopeConfig->getValue(self::RANGE);
            $startDateArr = date_parse($startDate);
            $createdAt = date('Y-m-d H:i:s', mktime(0,0,0,$startDateArr['month'],$startDateArr['day']+$i,$startDateArr['year']));
            $expDate = date('Y-m-d H:i:s', mktime(0,0,0,$startDateArr['month'],$startDateArr['day']+ $i +$range ,$startDateArr['year']));
            $coupons = $this->couponManagement->generate($this->generator);
            $coupon_id = $this->coupon->loadByCode($coupons)->getCouponId();
            $this->couponRepository->getById($coupon_id)->setExpirationDate($expDate)->setCreatedAt($createdAt)->save();
        }

        $this->messageManager->addSuccessMessage(__('%1 coupon(s) have been generated.', $i));
        $response = ['success' =>'Generated Birthday Coupon'];
        $result =  $this->resultFactory->create(ResultFactory::TYPE_JSON)->setData($response);

        return $result;


    }
}
